#
# listview.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#

require 'delta/view'


module Delta

  class ListView < View

    def initialize( ident, mbox )
      @identifier = ident
      @mbox = mbox
      @list = []
      @cache = []
      init_interface

      mbox.attach self
    end

    attr_reader :identifier

    CACHE_VERSION = 1

    def init2__parse
      @mbox.each do |port|
        add_port port
      end
    end

    def init2__cached( f )
      @list = Marshal.load(f)
      @cache = Marshal.load(f)
      @cache.each_with_index do |items, row|
        add_items row, items
      end
    end

    def init2__cache_failed
      @list = []
      @cache = []
    end

    def finalize__cache( f )
      Marshal.dump @list, f
      Marshal.dump @cache, f
    end

    def inspect
      "#<#{type} #{@mbox.inspect}>"
    end

    def mailbox
      @mbox
    end

    def size
      @list.size
    end

    def empty?
      @list.empty?
    end

    def add( port )
      return unless view_initialized?
      add_port port
    end

    def add_port( port )
      @list.push port
      @cache.push get_items(port)
      add_items @window.rows, @cache.last
    end

    def add_items( row, items )
      @window.append ['','','','']
      ### @window.set_pixmap row, 0, 
      @window.set_text row, 1, items[0]
      @window.set_text row, 2, items[1]
      @window.set_text row, 3, items[2]
    end

    def get_items( port )
      m = TMail::Mail.new(port)
      [
        m.strftime('%m/%d %H:%M', ''),
        m.friendly_from(''),
        getsubject(m)
      ]
    end

    def getsubject( m )
      m.subject('').sub( /\A(?:\s*\[[^]]+(\d+)?\])?(\s*(?:Re|SV):)*\s*/i ) {
          m = Regexp.last_match
          (m.begin(1) ? (m[1] + ' ') : '') +
          (m.begin(2) ? 'Re: ' : '')
      }
    end

    def del( port )
      return unless view_initialized?

      i = @list.index(port) or port_error! port
      @list.delete_at i
      @window.remove_row i
      if @focused >= @list.size then
        @window.select @list.size - 1
      end
      port
    end

    def clear
      return unless view_initialized?
      
      n = @list.size
      @list.clear
      @window.clear
      n
    end

    def move_selection_to( dest )
      # must be view_initialized
      @mbox.move_ports_to dest, selection()
    end

    def each_selection( &block )
      @window.each_selection(&block)
    end

    def selection
      set = []
      @window.each_selection do |idx|
        set.push @list[idx]
      end
      set
    end

    def init_interface
      @window = Gtk::CList.new( ['', 'Date', 'From', 'Subject'] )
      @window.set_usize 640, 320
      @window.set_column_width 0, 20
      @window.set_column_width 1, 100
      @window.set_column_width 2, 200
      @window.set_column_width 3, 320
      @window.selection_mode = Gtk::SELECTION_BROWSE

      @adjustment = Gtk::Adjustment.new(0,0,0,0,0,0)
      @window.vadjustment = @adjustment


      @focused = 0

      @window.signal_handler('select_row') {|w, row, col, event|
          next unless row
          @focused = row
          adjust_to @focused
          Delta.mailer.view focused
      }
    end

    attr_reader :window
    attr_reader :adjustment

    def begin_update
      @window.freeze
    end

    def end_update
      @window.thaw
    end

    def focused
      @list[ @focused ]
    end

    def focus_next
      @window.select @focused + 1
    end

    def focus_prev
      @window.select @focused - 1
    end

    def multiselect
      @window.selection_mode == Gtk::SELECTION_EXTENDED
    end

    def multiselect=( bool )
      if bool then
        @window.selection_mode = Gtk::SELECTION_EXTENDED
        Delta.mailer.message.freeze
      else
        @window.selection_mode = Gtk::SELECTION_BROWSE
        Delta.mailer.message.thaw
      end
    end

    def toggle_multiselect
      self.multiselect = !self.multiselect
    end

    def adjust_to( idx )
      adj = @adjustment

      #
      # NOTE: row must be visible
      #
      val = adj.value.to_i
      page = adj.page_size.to_i
      row = @window.row_height + 1   # +1 for spacing
      v = row * (idx + 3)
      if v > val + page then
# puts 'v'
        adj.value = ret = [v - page, adj.upper.to_i - page].min
      else
        v = row * (idx - 1)
        if v < val then
# puts '^'
          adj.value = ret = [v, adj.lower].max
        end
      end
# puts "adj.value=#{val} set=#{ret} row=#{row} page=#{page}"
    end
  
  end

end
