#
# mailer.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#

require 'nkf'
require 'tmail'
require 'gtk'
require 'delta/gtkadd'


module Delta

  @mailer = nil

  def Delta.set_mailer( m )
    @mailer and raise ArgumentError, 'Delta.mailer initialized twice'
    @mailer = m
  end

  def Delta.mailer
    @mailer
  end


  class Mailer_

    def initialize( cache_dir, mm )
      @cache_dir = cache_dir
      @menumaker = mm

      @views = ViewManager.new
      @views.add_current_view_changed_observer self
      # @views.add_view_added_observser self
      # @views.add_view_removed_observser self
      @profiles = {}
      @current_profile = nil
      @editor = nil
      @message = nil
    end

    #
    # mailbox view
    #

    attr_reader :views

    def on_current_view_changed( old, new )
      if old.multiselect then
        @multiple.clicked
      end
      
      unless new.window.realized? then
        new.init2 @cache_dir
        @summary_box.pack_start new.window, true, true, 0
      end
      @summary_scr.set_adjustment new.adjustment
      old.window.hide
      new.window.show_all
      if new.empty? then
        @notice.set_text "no mail"
      else
        @notice.set_text "#{new.size} mails"
      end
    end

    #
    # agents
    #

    def register_profile( name, i )
      @current_profile ||= i
      @profiles[name] = i
    end

    attr :profiles
    attr :current_profile

    def current_profile=( new )
      old, @current_profile = @current_profile, new
    end


    attr_accessor :editor

    def edit_new
      port = @current_profile.new_mail_composer.exec
      @editor.edit port or port.rm
    end

    def edit_reply( port )
      port = @current_profile.reply_mail_composer.exec( port )
      @editor.edit port or port.rm
    end

    def reedit( port )
      @editor.edit port
    end


    attr_reader :message

    def view( port )
      return unless @message
      @message.print port
    end

    #
    # main
    #

    def main
      create_widgets
      @win.show_all
      @views.current = @views.current   # force to paint

      begin
        Gtk.main
      ensure
        return unless @views
        @views.each_view do |v|
          v.detach
          ### v.finalize   # FIXME
        end
      end
    end

    #
    # create
    #

    def create_widgets
      #
      # create
      #

      @win = create_main_window()

      @summary_scr = Gtk::VScrollbar.new( @summary_adj )
      @summary_box = Gtk::VBox.new( false, 0 )

      @multiple = create_multiselect_button()
      @mboxcombo = create_mbox_combo()
      menu = create_menu()
      @notice = create_notice_label()

      message_adj = Gtk::Adjustment.new(0,0,0,0,0,0)
      message_scr = Gtk::VScrollbar.new( message_adj )
      message_win = Gtk::Text.new( nil, message_adj )
      @message = Message.new( message_win, message_adj )

      #
      # pack
      #

      pane = create_big_pane()
      @win.add pane

      upper_lev1_h = Gtk::HBox.new( false, 0 )
      pane.add1 upper_lev1_h
      pane.add2 message_win

      upper_lev2_v = Gtk::VBox.new( false, 0 )
      upper_lev1_h.pack_start upper_lev2_v, true, true, 0
      upper_lev1_h.pack_start message_scr, false, false, 2

      menubox = Gtk::HBox.new( false, 0 )
      menubox.pack_start @multiple, false, false, 4
      menubox.pack_start @mboxcombo, false, false, 0
      menubox.pack_start menu, true, true, 2

      summbox = Gtk::HBox.new( false, 0 )
      summbox.pack_start @summary_box, true, true, 0
      summbox.pack_start @summary_scr, false, false, 0

      upper_lev2_v.pack_start summbox, true, true, 0
      upper_lev2_v.pack_start menubox, false, false, 0
      upper_lev2_v.pack_start @notice, false, false, 0
    end

    def create_main_window
      win = Gtk::Window.new( Gtk::WINDOW_TOPLEVEL )
      win.set_title 'delta'
      win.border_width 0
      win.set_usize 640, 640
      win.signal_handler( 'destroy' ) {|w|
        Gtk.main_quit
      }
      win
    end

    def create_big_pane
      pane = Gtk::VPaned.new
      pane.handle_size = 7
      pane.gutter_size = 7
      pane
    end

    def create_multiselect_button
      msel = Gtk::CheckButton.new('')
      msel.signal_handler( 'toggled' ) {|w|
        @views.current.toggle_multiselect
      }
      msel
    end

    def create_mbox_combo
      combo = Gtk::Combo.new
      combo.set_usize 100, 24
      combo.entry.set_editable false
      combo.set_popdown_strings Delta.mailer.views.keys.sort
      combo.entry.set_text 'inbox'
      @cbpressing = false
      combo.button.signal_handler('pressed')  { @cbpressing = true }
      combo.button.signal_handler('released') { @cbpressing = false }
      combo.entry.signal_handler( 'changed' ) {|w|
          @views.select_view w.get_text  unless @cbpressing
      }
      combo
    end

    def create_notice_label
      label = Gtk::Label.new( 'delta ready' )
      label.set_usize 640, 20
      label.set_alignment 0, 0.5
      label
    end

    def create_menu
      @accel = Gtk::AccelGroup.new
      @accel.attach @win
      @menumaker.generate(@accel)
    end

  end

  class Mailer < Mailer_
  end


  class ViewManager

    def initialize
      @table = {}
      @current = nil

      @need_current_view_changed = []
      @need_view_added = []
      @need_view_removed = []
    end

    def []( name )
      @table[name]
    end

    def []=( name, view )
      raise ArgumentError, "view '#{name}' already exists" if @table.key? name
      @current ||= view
      @table[name] = view
      view_added view
    end

    def names
      @table.keys
    end

    alias keys names

    def to_a
      @table.to_a
    end

    def each_view( &block )
      @table.each_value(&block)
    end


    def select_view( name )   # useless?
      v = @table[name] or raise ArgumentError, "no such view: #{name}"
      self.current = v
    end

    attr_reader :current

    def current=( new )
      old, @current = @current, new
      current_view_changed old, new
      new
    end

    def add_current_view_changed_observer( recv )
      @need_current_view_changed.push recv
    end

    def current_view_changed( old, new )
      @need_current_view_changed.each do |recv|
        recv.__send__ :on_current_view_changed, old, new
      end
    end
    private :current_view_changed

    def add_view_added_observer( recv )
      @need_view_added.push recv
    end

    def view_added( view )
      @need_view_added.each do |recv|
        recv.__send__ :on_view_added, view
      end
    end
    private :view_added

    def add_view_removed_observer( recv )
      @need_view_removed.push recv
    end

    def view_removed( view )
      @need_view_removed.each do |recv|
        recv.__send__ :on_view_removed, view
      end
    end
    private :view_removed

  end

end
