#
# mbox.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#

module Delta

  class Mailbox
  
    def initialize( ld )
      @loader = ld
      @list = nil
      @views = []
    end

    def inspect
      "#<#{type} #{@loader.inspect}>"
    end

    def attach( view )
      @views.push view
    end

    def detach( view )
      @views.delete view or raise ArgumentError, "un-attached view: #{view}"
    end


    def each( &block )
      init
      # @loader.each_new_port {|port| @list.push port }
      @list.each(&block)
    end


    def new_port
      init

      port = @loader.new_port
      begin
        yield port if block_given?
      rescue
        begin
          port.rm
        rescue Errno::ENOENT
          ;
        end
        raise
      end

      updating {
        @list.push port
        @views.each do |v|
          v.add port
        end
      }

      port
    end

    alias new new_port

    def move_to( dest_mbox, port )
      init
      updating {
        _move_to dest_mbox, port
      }
    end

    def move_ports_to( dest_mbox, set )
      return 0 if set.empty?

      init
      updating {
        set.each do |port|
          _move_to dest_mbox, port
        end
      }
      set.size
    end

    def _move_to( dest_mbox, port )
      @views.each do |v|
        v.del port
      end
      @list.delete port

      dest_mbox.new {|dest|
        port.mv dest
      }
    end
    private :_move_to

    def remove( port )
      init
      updating {
        _remove port
      }
    end

    def remove_ports( set )
      init
      updating {
        set.each do |port|
          _remove port
        end
      }
    end
    
    def _remove( port )
      @views.each do |v|
        v.del port
      end
      @list.delete port
      port.rm
    end

    def clear
      init
      updating {
        @views.each do |v|
          v.clear
        end
        @list.each do |port|
          port.rm
        end
        n = @list.size
        @list.clear
        n
      }
    end

    private

    def init
      unless @list then
        @list = []
        @loader.each_port {|port| @list.push port }
      end
    end

    def updating
      begin
        @views.each do |v|
          v.begin_update
        end
        return yield
      ensure
        @views.each do |v|
          v.end_update
        end
      end
    end

  end

end
